from moncash.exceptions.http.connection_error import ConnectionError 
from moncash.exceptions.http.invalid_response_error import InvalidResponseError 
from moncash.exceptions.http.timeout_error import TimeoutError, ConnectTimeoutError, ReadTimeoutError



from moncash.exceptions.moncash_error import MoncashError 
from moncash.exceptions.configuration_error import ConfigurationError 
from moncash.exceptions.payment_error import PaymentError

from moncash.exceptions.authentication_error import AuthenticationError 
from moncash.exceptions.authorization_error import AuthorizationError 
from moncash.exceptions.gateway_timeout_error import GatewayTimeoutError 
from moncash.exceptions.request_timeout_error import RequestTimeoutError
from moncash.exceptions.server_error import ServerError
from moncash.exceptions.service_unavailable_error import ServiceUnavailableError
from moncash.exceptions.too_many_requests_error import TooManyRequestsError
from moncash.exceptions.unexpected_error import UnexpectedError
from moncash.exceptions.upgrade_required_error import UpgradeRequiredError
from moncash.exceptions.not_found_error import NotFoundError

from moncash.exceptions.http import *


from moncash.constants import API 
from moncash.exceptions import PaymentError

class Payment(object):
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config
    
    def __post(self, endpoint, payload):
        return self.config.http().post(endpoint, payload)
    
    def create(self, amount, reference):
        if amount == 0:
            raise PaymentError("Payment amount can not be zero")
        
        response = self.__post(
            endpoint = API[self.config.api_version]["create_payment"],
            payload = {
                "amount":amount,
                "orderId":reference
            } 
        )

        self.url = self.config.environment.redirect_url+"/Payment/Redirect?token="+response["payment_token"]["token"]

        return self.url
        
    def get_by_ref(self, reference):
        response = self.__post(
            endpoint=API[self.config.api_version]["get_payment_by_ref"],
            payload={
                    "orderId":reference
                }
        )

        return response['payment']

    def get_by_id(self, transactionId):
        response = self.__post(
            endpoint=API[self.config.api_version]["get_payment_by_id"],
            payload={
                    "transactionId":transactionId
                }
        )

        return response['payment']






import requests
import datetime
from base64 import encodebytes
import json 

from moncash.constants import API 

from moncash.exceptions import  (
                                    AuthenticationError,
                                    AuthorizationError,
                                    GatewayTimeoutError,
                                    ConnectionError,
                                    InvalidResponseError,
                                    ConnectTimeoutError,
                                    ReadTimeoutError,
                                    TimeoutError,
                                    NotFoundError,
                                    RequestTimeoutError,
                                    ServerError,
                                    ServiceUnavailableError,
                                    TooManyRequestsError,
                                    UnexpectedError,
                                    UpgradeRequiredError
                                )


class Http(object):

    @staticmethod
    def is_error_status(status):
        return status not in [200, 202, 201, 204, 422]

    @staticmethod
    def raise_exception_from_status(status, message=None):
        if status == 401:
            raise AuthenticationError()
        elif status == 403:
            raise AuthorizationError(message)
        elif status == 404:
            raise NotFoundError()
        elif status == 408:
            raise RequestTimeoutError()
        elif status == 426:
            raise UpgradeRequiredError()
        elif status == 429:
            raise TooManyRequestsError()
        elif status == 500:
            raise ServerError()
        elif status == 503:
            raise ServiceUnavailableError()
        elif status == 504:
            raise GatewayTimeoutError()
        else:
            raise UnexpectedError("Unexpected HTTP_RESPONSE " + str(status))
    
    def handle_exception(self, exception):
        if isinstance(exception, requests.exceptions.ReadTimeout):
            raise ReadTimeoutError(exception)
        elif isinstance(exception, requests.exceptions.ConnectTimeout):
            raise ConnectTimeoutError(exception)
        elif isinstance(exception, requests.exceptions.ConnectionError):
            raise ConnectionError(exception)
        elif isinstance(exception, requests.exceptions.HTTPError):
            raise InvalidResponseError(exception)
        elif isinstance(exception, requests.exceptions.Timeout):
            raise TimeoutError(exception)
        else:
            raise UnexpectedError(exception)


    def __init__(self, config, environment=None):
        self.config = config
        self.environment = environment or self.config.environment 

        self.__access_token = {"token":None, "taken_date":None}
    
    def __get_access_token(self):

        auth_string = b"Basic "+encodebytes(
            self.config.client_id.encode('ascii')+b":"+self.config.client_secret.encode('ascii')
        ).replace(b"\n", b"").strip()

        headers = {
            "Accept":"application/json",
            "Content-Type":"application/json",
            "Authorization": auth_string
        }

        resp = self.post(API[self.config.api_version]["auth"], headers_override=headers)

        access_token = resp["access_token"]

        self.__access_token["token"] =  'Bearer '+access_token
        self.__access_token["taken_date"] = datetime.datetime.now()

        return self.__access_token["token"]

    
    def __authorization_header(self):
        # check if we have an access token 
        if self.__access_token["token"]:
            # check if this access token is valid 
            if datetime.datetime.now() - self.__access_token["taken_date"] < 59:
                return self.__access_token["token"]
        
        return self.__get_access_token()

    def post(self, endpoint, payload=None, headers_override=None):
        params = {"scope":"read,write", "grant_type":"client_credentials"}

        if headers_override:
            headers = headers_override
        else:
             headers = {
                "Accept":"application/json",
                "Content-Type":"application/json",
                "Authorization":self.__authorization_header()
            }

        try:
            resp = requests.post(
                    self.config.base_url()+endpoint, 
                    params=params, 
                    headers=headers,
                    data=json.dumps(payload)
                )
        except Exception as e:
            self.handle_exception(e)

        resp_json = json.loads(resp.content.decode('utf-8'))

        if self.is_error_status(resp.status_code):
            self.raise_exception_from_status(resp.status_code)
 
        return resp_json
        


       
        

    


    
    
    







"""
                                  _     
  /\/\   ___  _ __   ___ __ _ ___| |__  
 /    \ / _ \| '_ \ / __/ _` / __| '_ \ 
/ /\/\ \ (_) | | | | (_| (_| \__ \ | | |
\/    \/\___/|_| |_|\___\__,_|___/_| |_|
                                        

This is an unofficial wrapper providing convenient access to 
the MonCash API for applications written in Python

:copyright: (c) 2020-2022 by DOKLA.
:license: Apache 2.0, see LICENSE for more details.
"""


from moncash.configuration import Configuration
from moncash.payment import Payment

class Moncash(object):

    def __init__(self, config=None, **kwargs):
        if isinstance(config, Configuration):
            self.config = config 
        else:
            self.config = Configuration(
                client_id=kwargs.get("client_id"),
                client_secret=kwargs.get("client_secret"),
                environment=kwargs.get("environment")
            ) 
        self.payment = Payment(self)
        

if __name__ == '__main__':
    print("__package__")
    
        


    




from moncash.exceptions.unexpected_error import UnexpectedError 

class TimeoutError(UnexpectedError):
    pass

class ConnectTimeoutError(TimeoutError):
    pass

class ReadTimeoutError(TimeoutError):
    pass



BASE = "moncashbutton.digicelgroup.com"

SANDBOX_HOST = f"sandbox.{BASE}/Api"

PROD_HOST = f"{BASE}/Api"

SANDBOX_REDIRECT_URL = f"https://sandbox.{BASE}/Moncash-middleware"
PROD_REDIRECT_URL = f"https://{BASE}/Moncash-middleware"

API = {
    "v1": {
        "auth": "/oauth/token",
        "create_payment": "/v1/CreatePayment",
        "transfert":"v1/Transfert",
        "get_payment_by_id": "/v1/RetrieveTransactionPayment",
        "get_payment_by_ref": "/v1/RetrieveOrderPayment"
    }
}



"""
                                  _     
  /\/\   ___  _ __   ___ __ _ ___| |__  
 /    \ / _ \| '_ \ / __/ _` / __| '_ \ 
/ /\/\ \ (_) | | | | (_| (_| \__ \ | | |
\/    \/\___/|_| |_|\___\__,_|___/_| |_|
                                        

This is an unofficial wrapper providing convenient access to 
the MonCash API for applications written in Python

:copyright: (c) 2020 by DOKLA.
:license: Apache 2.0, see LICENSE for more details.
"""

from moncash.gateway import Moncash 
from moncash.configuration import Configuration
from moncash.environment import Environment 


from setuptools import setup, find_packages

setup(
    name='moncash',
    version='1.0.3',
    license='MIT',
    description='Simple python wrapper to perform and retrieve payment to Digicel Moncash with Python',
    packages=find_packages('src'),
    package_dir={'': 'src'}, 
    author="Dokla",
    author_email="support@dokla.ht",
    install_requires=["requests>=2.25.1,<3.0"],
)


from moncash import constants

class Environment(object):
    
    def __init__(self, name, host):
        self.__name__ = name 
        self.host = host
        self.protocol = "https://"

        if name == 'Sandbox':
            self.redirect_url = constants.SANDBOX_REDIRECT_URL 
        elif name == 'Production':
            self.redirect_url = constants.PROD_REDIRECT_URL
        else:
            raise EnvironmentError("Environment should be named 'Sandbox' or 'Production'")
        

Sandbox = Environment('Sandbox', constants.SANDBOX_HOST)

Production = Environment('Production', constants.PROD_HOST)



from moncash.http import Http

class Configuration(object):
    def __init__(self, client_id=None, client_secret=None, environment=None):

        self.client_id = client_id
        self.client_secret = client_secret
        self.environment = environment

        self.api_version = "v1"
    
    def http(self):
        return Http(self)
    
    def base_url(self):
        return self.environment.protocol+self.environment.host
    